/* statinfo.c - demonstrates using stat() to obtain
 *              file information.
 *            - some members are just numbers...
 */
#include <stdio.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <time.h>

void show_stat_info(char *, struct stat *);

int main(int ac, char *av[])
{
    struct stat info;        /* buffer for file info */
	int status;
	char *name;

    if (ac>1) {
		name = av[1];
		status = lstat(name, &info);
	}
	else {
		name = "stdin";
		status = fstat(0, &info);
	}
	if (status == -1)
	{
		perror(name);
		return 1;
	}
    show_stat_info( name, &info );
	return 0;
 }
char *get_filetype(int mode)
{
	if (S_ISREG(mode)) return "regular file";
	if (S_ISDIR(mode)) return "directory";
	if (S_ISCHR(mode)) return "character device";
	if (S_ISBLK(mode)) return "block device";
	if (S_ISFIFO(mode)) return "FIFO or pipe";
	if (S_ISLNK(mode)) return "symbolic link";
	if (S_ISSOCK(mode)) return "socket";
	return "unrecognized filetype";
}

void show_stat_info(char *fname, struct stat *buf)
/*
 * displays some info from stat in a name=value format
 */
{
    printf("   name: %s\n", fname );               /* filename    */
    printf("   mode: %o\n", buf->st_mode);         /* type + mode */
	printf("   type: %s\n", get_filetype(buf->st_mode));
	printf("  inode: %d\n", (int) buf->st_ino);    /* file serial number */
	printf("    dev: %d\n", (int) buf->st_dev);    /* device */
    printf("  links: %d\n", buf->st_nlink);        /* # links     */
    printf("   user: %d\n", buf->st_uid);          /* user id     */
    printf("  group: %d\n", buf->st_gid);          /* group id    */
    printf("   size: %ld\n", buf->st_size);         /* file size   */
	printf(" blocks: %d\n", (int) buf->st_blocks); /* # 512 byte blocks */
	printf("blksize: %d\n", (int) buf->st_blksize); /* optimal block size */
    printf("modtime: %s",   ctime(&buf->st_mtime));        /* modified    */
}
